#include <windows.h>
#include <mmsystem.h>  // timer 
#pragma comment(lib,"winmm.lib") 

BOOL useQPF=FALSE;
LONGLONG cur_time=0,last_time=0;
float elapsed=0,time_scale=0;

void QPFTimer()
{
	if(QueryPerformanceFrequency((LARGE_INTEGER *)&cur_time))
	{
		time_scale=1.0f/cur_time;
		QueryPerformanceCounter((LARGE_INTEGER *)&last_time);
		useQPF=TRUE;
	}
	else
	{
		time_scale=0.001f;
		last_time=timeGetTime();
		useQPF=FALSE;
	}
}

float GetElapsed()
{
	if(useQPF) QueryPerformanceCounter((LARGE_INTEGER *)&cur_time);
	else cur_time=timeGetTime();
	elapsed=(cur_time-last_time)*time_scale;
	last_time=cur_time;

	return elapsed;
}

typedef enum _D3DAPPTIMER{
	TIMER_START,TIMER_STOP,TIMER_ADVANCE,TIMER_GETAPPTIME,TIMER_GETELAPSEDTIME,
	TIMER_GETABSOLUTETIME,TIMER_RESET,
} D3DAPPTIMER;

extern BOOL ready;
BOOL FrameMoving;
FLOAT Timer(D3DAPPTIMER);

VOID Pause(BOOL Pause)
{
	static DWORD apc=0L; 

	apc+=(Pause ? +1 : -1);
	ready=(apc ? FALSE : TRUE);

	if(Pause && (1==apc) )
	{
		if(FrameMoving)
			Timer(TIMER_STOP);
	}

	if(0==apc)
	{
		if(FrameMoving)
			Timer(TIMER_START);
	}
}

FLOAT Timer(D3DAPPTIMER command)
{
	static BOOL init=FALSE;
	static BOOL useQPF=FALSE;
	static LONGLONG QPFtps=0;

	if(FALSE==init)
	{
		init=TRUE;
		LARGE_INTEGER lFreq;
		useQPF=QueryPerformanceFrequency(&lFreq); // returns 0 if not supported 
		if(useQPF)
			QPFtps=lFreq.QuadPart;
	}

	if(useQPF)
	{
		static LONGLONG fStopTime=0;
		static LONGLONG fLastElapsedTime=0;
		static LONGLONG fBaseTime=0;
		double fTime;
		double fElapsedTime;
		LARGE_INTEGER lTime;

		if(fStopTime !=0 && command !=TIMER_START && command !=TIMER_GETABSOLUTETIME)
			lTime.QuadPart=fStopTime;
		else 
			QueryPerformanceCounter(&lTime);

		if(command==TIMER_GETELAPSEDTIME)
		{
			fElapsedTime=(double)(lTime.QuadPart-fLastElapsedTime) / (double) QPFtps;
			fLastElapsedTime=lTime.QuadPart;
			return (FLOAT)fElapsedTime;
		}

		if(command==TIMER_GETAPPTIME)
		{
			double fAppTime=(double)(lTime.QuadPart-fBaseTime) / (double) QPFtps;
			return (FLOAT)fAppTime;
		}

		if(command==TIMER_RESET)
		{
			fBaseTime=lTime.QuadPart;
			fLastElapsedTime=lTime.QuadPart;
			return 0.0f;
		}

		if(command==TIMER_START)
		{
			fBaseTime+=lTime.QuadPart-fStopTime;
			fStopTime=0;
			fLastElapsedTime=lTime.QuadPart;
			return 0.0f;
		}

		if(command==TIMER_STOP)
		{
			fStopTime=lTime.QuadPart;
			fLastElapsedTime=lTime.QuadPart;
			return 0.0f;
		}

		if(command==TIMER_ADVANCE)
		{
			fStopTime+=QPFtps/10;
			return 0.0f;
		}

		if(command==TIMER_GETABSOLUTETIME)
		{
			fTime=lTime.QuadPart/(double) QPFtps;
			return (FLOAT)fTime;
		}

		return -1.0f;
	}
	else
	{
		// use timeGetTime()
		static double fLastElapsedTime=0.0;
		static double fBaseTime=0.0;
		static double fStopTime=0.0;
		double fTime;
		double fElapsedTime;

		if(fStopTime !=0.0 && command !=TIMER_START && command !=TIMER_GETABSOLUTETIME)
			fTime=fStopTime;
		else
			fTime=timeGetTime() * 0.001;

		if(command==TIMER_GETELAPSEDTIME)
		{
			fElapsedTime=(double) (fTime-fLastElapsedTime);
			fLastElapsedTime=fTime;
			return (FLOAT) fElapsedTime;
		}

		if(command==TIMER_GETAPPTIME)
		{
			return (FLOAT)(fTime-fBaseTime);
		}

		if(command==TIMER_RESET)
		{
			fBaseTime=fTime;
			fLastElapsedTime=fTime;
			return 0.0f;
		}

		if(command==TIMER_START)
		{
			fBaseTime+=fTime-fStopTime;
			fStopTime=0.0f;
			fLastElapsedTime=fTime;
			return 0.0f;
		}

		if(command==TIMER_STOP)
		{
			fStopTime=fTime;
			return 0.0f;
		}

		if(command==TIMER_ADVANCE)
		{
			fStopTime+=0.1f;
			return 0.0f;
		}

		if(command==TIMER_GETABSOLUTETIME)
		{
			return (FLOAT)fTime;
		}

		return -1.0f;
	}
}


